#
# Module grouping the procedures for "quantitative" analysis of 
# interaction graphs 
#
# - simulation
#

proc run_simulation { sim_length } {
    global KEEP

    set TMP_GRAPH_FILE "/tmp/gred[pid].adj"
    set TMP_RES_FILE "/tmp/gred[pid].res"

    if [write_matrix $TMP_GRAPH_FILE sparse] {
	exec gr_sim -f $TMP_GRAPH_FILE -nevt $sim_length -compile \
	    | grep gamma > $TMP_RES_FILE
	
	set resfile [open $TMP_RES_FILE r]
	set result [read $resfile]
	close $resfile

	LogResult "Simulation Results" "$result"

    }

    if !$KEEP {
	RemoveFile $TMP_GRAPH_FILE
	RemoveFile $TMP_RES_FILE
    }
}

proc simulate { } {

    global sim_length gen_file_name

    if [catch {toplevel .params}] { return }

    wm title .params "Simulation Parameters"

    frame .params.simlength
    label .params.simlength.label -text "Simulation Length" -width 15 -anchor w
    entry .params.simlength.field -textvar sim_length -width 8 -relief sunken
    
    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n run_simulation \$sim_length"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"
    
    pack .params.simlength.label .params.simlength.field -side left \
	-fill x
    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.simlength -side top -fill x
    pack .params.actions -side top

}
    
proc Compute_Bounds { } {
    global KEEP

    set TMP_GRAPH_FILE "/tmp/gred[pid].adj"
    set TMP_RES_FILE "/tmp/gred[pid].res"

    if [write_matrix $TMP_GRAPH_FILE sparse] {
	# there max be "warnings" in the result. grepping them out...
	exec gr_bound -f $TMP_GRAPH_FILE \
	    | grep -v "gr_bound" > $TMP_RES_FILE

	set resfile [open $TMP_RES_FILE r]
	set result [read $resfile]
	set result [split $result]
	close $resfile

	LogResult "Bounds on the growth rate" \
	    "Lower bound: [lindex $result 0]\nUpper bound: [lindex $result 1]\n"
	
    }

    if !$KEEP {
	RemoveFile $TMP_GRAPH_FILE
	RemoveFile $TMP_RES_FILE
    }
}

proc Compute_Matrix_Bounds { } {
    global node_list
    global KEEP

    set TMP_MAT_FILE "/tmp/gred[pid].mat"
    set TMP_RES_FILE "/tmp/gred[pid].res"

    if [write_maxplus_matrices $TMP_MAT_FILE sparse "A"] {
	# computing number depth of the tree in mat_bound, estimating
	# 50.000 matrices / sec and a waiting time of 5s. 
	set N [llength $node_list]
	set Kmax [expr int(log(5*50000) / log($N))]
	puts "Depth = $Kmax"
	exec mat_bound -f $TMP_MAT_FILE -exact -nmax $Kmax > $TMP_RES_FILE

	set resfile [open $TMP_RES_FILE r]
	set result [read $resfile]
 	set result [split $result]
	close $resfile

	LogResult "Bounds on the growth rate" \
	    "Lower bound: [lindex $result 0]\nUpper bound: [lindex $result 1]\n"
    }

    if !$KEEP {
	RemoveFile $TMP_MAT_FILE
	RemoveFile $TMP_RES_FILE
    }
}

proc Is_Empty_Graph { } {
    global node_list

    return [expr [llength $node_list] == 0]
}
