#
# Module of procedures manipulating the selection
#

proc Select_Object { win cur_obj } {
    global DEBUG

    if { "" != $cur_obj } {
	if { [Is_Type $cur_obj NODE] || [Is_Type $cur_obj ARC] } {
	    Toggle_Selection $win $cur_obj
	}
    }
}

proc Toggle_Selection { win obj } {

    set objtags  [$win gettags $obj]

    if { -1 != [lsearch -exact $objtags "selected"] } {
	# already selected. Unselect it
	$win delete [$win find withtag "Sel$obj"]
	# remove selection tag
	set idx [lsearch -exact $objtags selected]
	set objtags [lreplace $objtags $idx $idx]
	$win itemconfigure $obj -tags $objtags
    } else {
	set bbox [$win bbox $obj]
	set tlist [list SELECT "Sel$obj"]
	set newid [eval {$win create rectangle} $bbox \
		       -outline goldenrod \
		       -width 5 -tags {$tlist}]
	$win itemconfigure $obj -tags [concat $objtags selected]
	$win lower $newid $obj
    }
}

proc Select_Current_Object { win } {
    global DEBUG

    set cur_obj [$win find withtag current]
    if $DEBUG {
	puts "Current object in $win = $cur_obj"
    }

    Select_Object $win $cur_obj
}

proc Delete_Selection { win } {
    global NUMBERS DEBUG

    set targets {}

    # find objects in selection zone(s)
    set zones [$win find withtag SELZONE]
    foreach zone $zones {
	set bbox [$win bbox $zone]
	eval {lappend targets} [eval {$win find enclosed} $bbox]
    }
    puts "Zone targets: $targets"
    eval {lappend targets} [$win find withtag selected]

    foreach obj_id $targets {
	Delete_Object $win $obj_id
    }

    #destroy selection zones
    $win delete withtags SELZONE

    if $NUMBERS {
	Update_Numbers $win
    }    
}

proc clique_selection { win } {
    global node_list arc_list

    set thelist [$win find withtag NODE]
    set select {}
    set select_ids {}

    # find selected nodes and their ids
    foreach node $node_list {
	set node_id [lindex $node 0]
	set state [$win itemconfigure $node_id -fill ]
	set state [lindex $state 4]
	if { "green" == $state } {
	    set select [lappend select $node]
	    set select_ids [lappend select_ids $node_id]
	}
    }

    puts -nonewline "Making complete graph with ids: $select_ids"

    # make new (?) arcs
    for { set i 0 } { $i < [llength $select_ids]} { incr i 1 } {
	for { set j [expr $i+1] } { $j < [llength $select_ids] } { incr j } {

	    set id1 [lindex $select_ids $i]
	    set id2 [lindex $select_ids $j]
	    set node1 [lindex $select $i]
	    set node2 [lindex $select $j]
	    set xpos1 [lindex $node1 1]
	    set ypos1 [lindex $node1 2]
	    set xpos2 [lindex $node2 1]
	    set ypos2 [lindex $node2 2]

	    set newid [$win create \
			       line $xpos1 $ypos1 $xpos2 $ypos2 -tag ARC ]

	    set new_arc [list $newid $id1 $id2]
	    set arc_list [lappend arc_list $new_arc]

	}
    }

    # reset selection to none
    Cancel_All_Selections $win

    puts " ...done."

}

proc Select_Zone { win xpos ypos } {
    global cur_sel_zone
    
    puts -nonewline "Current zone = $cur_sel_zone"

    switch [llength $cur_sel_zone] {
	0
	{
	    # begin the creation of a selection zone
	    set cur_sel_zone [list $xpos $ypos]
	}
	2 {
	    # finish the creation and display 
	    lappend cur_sel_zone $xpos $ypos
	    # eval {$win create rectangle} $cur_sel_zone \
# 		-outline goldenrod -width 5 -tags SELZONE
	    set listobj [eval {$win find enclosed} $cur_sel_zone]
	    if { [llength $listobj] > 0 } {
		puts "Zone encloses objs $listobj"
		set cur_sel_zone [eval {$win bbox} $listobj]
		puts "Made rectangle at: $cur_sel_zone"
		eval {$win create rectangle} $cur_sel_zone \
		    -stipple gray50 \
		    -outline goldenrod -width 2 -tags SELZONE
	    }
	}
	4 {
	    # remove current selection and initiate a new one
	    $win delete [$win find withtag SELZONE]
	    set cur_sel_zone [list $xpos $ypos]
	}
	default {
	    puts "Internal Error: bad length for selection list."
	    set cur_sel_zone {}
	}
    }

    puts " --> $cur_sel_zone"

}

######################################################################
# Select_Object_List -
# Procedure for selecting a list of objects. Usually, this is used
# for highlighting a list. For instance with the "bad" objects of
# qualitative analysis. Perhaps however the "highlight" and "select"
# functionalities should be separated.
#
# Parameters:
# - win		the canvas widget where the objects are
# - obj_list	the list of objects
#
proc Select_Object_List { win obj_list } {
    global DEBUG
    
    Cancel_All_Selections $win
    foreach obj $obj_list {
	if $DEBUG {
	    puts "Tog $obj"
	}
	Toggle_Selection $win $obj
    }
}

######################################################################
# Select_All -
# Procedure to select all objects in the canvas.
#
# Parameters:
# - win		the canvas widget where the objects are
#
proc Select_All { win } {

    set cur_sel_zone [$win bbox all]
    eval {$win create rectangle} $cur_sel_zone \
	-stipple gray50 \
	-outline goldenrod -width 2 -tags SELZONE
}

######################################################################
# Cancel_All_Selections -
# Procedure to remove all object selections in the canvas.
#
# Parameters:
# - win		the canvas widget where the objects are
#
proc Cancel_All_Selections { win } {

    set sellist [$win find withtag selected]
    foreach obj $sellist {
	Toggle_Selection $win $obj
    }
}
