#
# Module for the management of edit/select/delete modes (and others)
# 
######################################################################
# Reset_Standard_Bindings -
# Procedure to restore the bindings that prevail out of any special
# mode. 
#
# Parameters:
# - win		the widget (normally: the graph canvas) to bind
#
proc Reset_Standard_Bindings { win } {
    
    $win bind NODE <1> {select_node on %W}
    $win bind NODE <2> {select_node off %W}
    $win bind NODE <3> { }

    $win bind PLACE <KeyPress-plus> {Increment_Mark %W 1}
    $win bind PLACE <KeyPress-minus> {Increment_Mark %W -1}
    $win bind PLACE <KeyRelease> {puts "%%K"}

    bind $win <Any-Button> {print_coords %W %x %y}
    # bind $win <1> {Select_Object %W}
    bind $win <1> { }
    bind $win <2> { }
    bind $win <3> {Cancel_All_Selections %W}
    # bind $win <KeyPress> {puts stdout {%%K=%K %%A=%A}}
    bind $win <KeyPress-plus> {Increment_Mark %W 1}
    bind $win <KeyPress-minus> {Increment_Mark %W -1}
    bind $win <KeyPress-Delete> {Delete_Current_Object %W}
    bind $win <KeyPress-s> {Select_Object %W}
    bind $win <KeyPress-a> {Edit_Current_Object_Attributes %W %X %Y}
    bind $win <KeyPress-f> {Fire_Current_Node %W 1}
    bind $win <KeyPress-u> {Fire_Current_Node %W -1}
    bind $win <KeyPress-r> {Recycle_Transition %W}

    $win configure -cursor ""

    bind .topmenu.export <Enter> {Update_Export_Menu}
    bind .topmenu.quanti <Enter> {Update_Quantitative_Menu}
}

######################################################################
# Enter_Mode -
# Procedure handling the start of a mode: text written in the help mode
# area, appearance of buttons, cursors and so on.
# If the current mode is not "none", then it is exited before by a call
# to Exit_Mode.
#
# Parameters:
# - mode	the new mode in which to enter
#
proc Enter_Mode { mode } {
    global cur_mode Information Mode img_dir
    global DEBUG

    if $DEBUG {
	puts "Entering mode $mode"
    }

    # win not passed as parameter for now. But might be one day...
    set win .graph.canvas

    if { 0 != [string compare $cur_mode "none"] } {
	Exit_Mode $cur_mode
    }

    # operations common to all changes of modes
    if [winfo exists .edit.$mode] {
	.edit.$mode configure -relief sunken
    }
    # default rebinding.
    bind $win <3> "Exit_Mode $mode"
    bind $win <2> "Exit_Mode $mode"
    bind $win <Escape> "Exit_Mode $mode"

    # operations that are mode-specific.
    switch $mode {
	place {
	    $win configure -cursor "@$img_dir/place.xbm black"
	    bind $win <1> { Put_Place %W [%W canvasx %x] [%W canvasy %y] }
	    set Mode "Place"
	    set Information "\[l\] Put Place \[c\] Quit mode \[r\] Quit mode"
	}
	trans {
	    $win configure -cursor "@$img_dir/trans.xbm black"
	    bind $win <1> { Put_Trans %W [%W canvasx %x] [%W canvasy %y] }
	    set Mode "Transition"
	    set Information "\[l\] Put Transition \[c\] Quit mode \[r\] Quit mode"
	}
	link {
	    $win configure -cursor ""
	    bind $win <1> { Start_Link %W [%W canvasx %x] [%W canvasy %y] }
	    bind $win <2> { Abort_Link %W }
	    set Mode "Link"
	    set Information "\[l\] Start/Stop Link \[c\] Abort Link \[r\] Quit mode"
	}
	poiss {
	    $win configure -cursor "@$img_dir/source.xbm blue"
	    bind $win <1> { Put_Source %W [%W canvasx %x] [%W canvasy %y] }
	    set Mode "Poisson"
	    set Information "\[l\] Put Poisson Source \[c\] Quit mode \[r\] Quit mode"
	}
	info {
	    $win configure -cursor question_arrow
	    bind $win <1> { Edit_Current_Object_Attributes %W %X %Y }
	    set Mode "Info"
	    set Information "\[l\] Inspect Object Attributes \[c\] Quit mode \[r\] Quit mode"
	}
	move {
	    $win configure -cursor target
	    bind $win <1> { Pick_Object_Or_Zone %W } 
	    bind $win <2> { Drop_Object_Or_Zone %W [%W canvasx %x] [%W canvasy %y] } 
	    bind $win <Right> { Move_Object_Or_Zone %W 5 0}
	    bind $win <Left>  { Move_Object_Or_Zone %W -5 0}
	    bind $win <Up>    { Move_Object_Or_Zone %W 0 -5}
	    bind $win <Down>  { Move_Object_Or_Zone %W 0 5}
	    bind $win <Shift-Right> { Move_Object_Or_Zone %W 20 0}
	    bind $win <Shift-Left>  { Move_Object_Or_Zone %W -20 0}
	    bind $win <Shift-Up>    { Move_Object_Or_Zone %W 0 -20}
	    bind $win <Shift-Down>  { Move_Object_Or_Zone %W 0 20}
	    bind $win <Control-Right> { Move_Object_Or_Zone %W 1 0}
	    bind $win <Control-Left>  { Move_Object_Or_Zone %W -1 0}
	    bind $win <Control-Up>    { Move_Object_Or_Zone %W 0 -1}
	    bind $win <Control-Down>  { Move_Object_Or_Zone %W 0 1}

	    set Mode "Move"
	    set Information "\[l\] Pick \[c\] Put \[r\] Quit mode"
	}
	copy {
	    $win configure -cursor target
	    bind $win <1> { Pick_Object_Or_Zone %W } 
	    bind $win <2> { Copy_Object_Or_Zone %W [%W canvasx %x] [%W canvasy %y] } 
	    set Mode "Copy"
	    set Information "\[l\] Pick \[c\] Put \[r\] Quit mode"
	}
	del {
	    $win configure -cursor pirate
	    bind $win <1> { Delete_Current_Object %W }
	    set Mode "Delete"
	    set Information "\[l\] Delete \[c\] Quit mode \[r\] Quit mode"
	}
	sel {
	    $win configure -cursor target
	    bind $win <1> { Select_Object %W }
	    bind $win <2> { Select_Zone %W [%W canvasx %x] [%W canvasy %y]}
	    set Mode "Select"
	    set Information "\[l\] Select Object \[c\] Select Zone \[r\] Quit mode"
	}
	default {
	    ReportError "Mode Error" "unregistered mode: $mode"
	    return
	}
    }

    set cur_mode $mode
}

######################################################################
# Exit_Mode -
# Procedure handling the exit of a mode: text written in the help mode
# area, appearance of buttons, cursors and so on.
#
# Parameters:
# - mode	the mode to exit.
#
proc Exit_Mode { mode } {
    global cur_mode Information Mode curlink
    global cur_link cur_sel_zone cur_picked_obj
    global DEBUG

    # win not passed as parameter for now. But might be one day...
    set win .graph.canvas

    if $DEBUG {
	puts "Exiting mode $mode"
    }

    # operations common to all modes
    Reset_Standard_Bindings $win
    set Information ""
    set Mode ""
    .edit.$mode configure -relief raised

    # operations that are mode-specific.
    switch $mode {
	link { 
	    if [info exists curlink] {
		$win delete $curlink
	    }
	}
	copy -
	move {
	    if [info exists cur_picked_obj] {
		$win itemconfigure $cur_picked_obj -outline black
		unset cur_picked_obj
	    }
	    $win delete [$win find withtag SELZONE]
	}
	sel { 
	    if { [llength $cur_sel_zone] != 4 } {
		set cur_sel_zone {}
	    }
	}
    }
   
    # finish
    set cur_mode none
}
