#
# Standard tetris sets creation module for teted
#
######################################################################

proc Control_Panel { win } {
    global create_props

    frame $win.piece
    label $win.piece.label  -text "Piece Weight" -width 20 -anchor w
    entry $win.piece.weight -textvar create_props(weight) \
	-width 8 -relief sunken
    pack $win.piece.label $win.piece.weight -side left
    pack $win.piece -side top -fill x

    return [list $win.piece]
}

proc Make_Cycle { } {

    global size

    if [catch {toplevel .params}] { return }

    wm title .params "Cycle Parameters"

    frame .params.size
    label .params.size.label -text "Number of Pieces:" -width 15 -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken
    frame .params.chord
    label .params.chord.label -text "List of chords:" -width 15 -anchor w
    entry .params.chord.field -textvar chordlist -width 10 -relief sunken

    pack .params.size.label .params.size.field -side left -fill x
    pack .params.chord.label .params.chord.field -side left -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n\create_cycle \$size \$chordlist"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.size .params.chord .params.actions -side top

}

proc create_cycle { size chordlist } {
    global cur_width cur_piece_nb 
    global DEBUG

    if { $size <= 0 } { return }

    set cur_width $size
    Make_Canvas
    
    Empty_Graph .graph
    Empty_Store

    set chordlist [Trim_Chordlist $size $chordlist]
    if { $DEBUG } {
	puts -nonewline \
	    "Creating cycle with $size pieces and chords $chordlist..."
	puts "([llength $chordlist] chords)"
    }

    # Creation of the chords
    for { set j 0 } { $j < [llength $chordlist] } {incr j 1} {

	set chord_length [lindex $chordlist $j]
	if $DEBUG {
	    puts "doing chord $chord_length"
	}

	for { set i 0 } { $i < $size } { incr i 1 } {
	    # the list of chords is "trimmed" but the case of size/2
	    # is to be handled here.
	    if [expr { 2*$chord_length != $size } \
		    || { $i < $chord_length } ] {
		# re-initializing profiles
		for { set k 0 } { $k < $size } { incr k 1 } {
		    set LowProf($k) eps
		    set HiProf($k) eps
		}

		# making piece
		set LowProf($i) 0
		set HiProf($i) 1

		set k [expr ($i+$chord_length) % $size]
		set LowProf($k) 0
		set HiProf($k) 1

		# warning! when implementing multiple chords, remember about 
		# diagonal ones...

		if $DEBUG {
		    puts "Made = [list [array get LowProf] [array get HiProf]]"
		}
		set newpiece [list [array get LowProf] [array get HiProf]]
		New_Piece
		Store_Piece $newpiece 0
	    }
	}
	
    }

    # now the colors
    Create_Colors $cur_piece_nb

    # finally, redisplay the store (is it really necessary?)...
    Display_Store 1 $cur_piece_nb

    if $DEBUG {
	puts " ...Done"
    }

}

proc Make_String { } {

    global size

    if [catch {toplevel .params}] { return }

    wm title .params "Line Parameters"

    frame .params.size
    label .params.size.label -text "Number of Columns:" -width 20 -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken
    frame .params.chord

    pack .params.size.label -side left -anchor w
    pack .params.size.field -side left -fill x -expand true

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n\create_string \$size"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.size .params.chord .params.actions -side top

}

proc create_string { size } {
    global cur_width cur_piece_nb 
    global DEBUG

    if { $size <= 0 } { return }

    set cur_width $size
    Make_Canvas
    
    Empty_Graph .graph
    Empty_Store

    if { $DEBUG } {
	puts -nonewline \
	    "Creating line with $size columns and [expr $size-1] pieces."
    }

    for { set i 0 } { $i < [expr $size-1] } { incr i 1 } {
	# re-initializing profiles
	for { set k 0 } { $k < $size } { incr k 1 } {
	    set LowProf($k) eps
	    set HiProf($k) eps
	}

	# making piece
	set LowProf($i) 0
	set HiProf($i) 1

	set LowProf([expr $i+1]) 0
	set HiProf([expr $i+1]) 1

	set newpiece [list [array get LowProf] [array get HiProf]]
	New_Piece
	Store_Piece $newpiece 0
    }

    # now the colors
    Create_Colors $cur_piece_nb

    # finally, redisplay the store (is it really necessary?)...
    Display_Store 1 $cur_piece_nb

    if $DEBUG {
	puts " ...Done"
    }

}

proc Make_Complete { } {

    global size

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of Nodes:"
    entry .params.size.field -textvar size -width 3 -relief sunken

    pack .params.size.label .params.size.field -side left

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n\create_cycle \$size \[iota 1 \[expr \$size/2\]\]"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.size .params.actions -side top

}

proc create_star { size } {
    global cur_width cur_piece_nb 
    global DEBUG

    if { $size < 0 } { return }

    set cur_width $size
    Make_Canvas
    
    Empty_Graph .graph
    Empty_Store

    if $DEBUG {
	puts -nonewline "Creation of a star with $size branches"
    }

    # making the "bar"
    for { set i 0 } { $i < $size } { incr i 1 } {
	set LowProf($i) 0
	set HiProf($i) 1
    }

    New_Piece
    Store_Piece [list [array get LowProf] [array get HiProf]] 0

    # making the other ones
    for { set i 0 } { $i < $size } { incr i 1 } {
	set LowProf($i) eps 
	set HiProf($i) eps
    }
    for { set i 0 } { $i < $size } { incr i 1 } {
	set LowProf($i) 0
	set HiProf($i) 1

	New_Piece
	Store_Piece [list [array get LowProf] [array get HiProf]] 0
	    
	set LowProf($i) eps
	set HiProf($i) eps
    }

    # now the colors
    Create_Colors $cur_piece_nb

    # finally, redisplay the store...
    Display_Store 1 $cur_piece_nb

    if $DEBUG {
	puts " ...Done"
    }
}

proc Make_Star { } {

    global size

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of branches:" -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken

    pack .params.size.label .params.size.field -side left

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n\create_star \$size"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.size .params.actions -side top

}

proc create_random { size } {
    global cur_width cur_piece_nb
    global DEBUG

    randomInit 1

    if { $size <= 0 } { return }

    if { $DEBUG } {
	puts -nonewline "Creation of a random Tetris game with $size pieces"
    }

    set cur_width $size
    Make_Canvas
    
    Empty_Graph .graph
    Empty_Store

    # je ne sais pas ce qu'est un jeu de Tetris "aleatoire". J'arrete
    # ici pour le moment...
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos1 [xcoord $xorig $Radius [expr $i*$Alpha]]
	set ypos1 [ycoord $yorig $Radius [expr $i*$Alpha]]

	for { set j [expr $i+1] } { $j < $size } { incr j 1 } {
	    if { [random] < $create_props(proba) } {
		
		set xpos2 [xcoord $xorig $Radius [expr $j*$Alpha]]
		set ypos2 [ycoord $yorig $Radius [expr $j*$Alpha]]

		set end1 [clip $xpos2 $ypos2 $xpos1 $ypos1 $node_radius]
		set end2 [clip $xpos1 $ypos1 $xpos2 $ypos2 $node_radius]

		set newid [eval {.graph create line} $end1 $end2 -tag ARC]

		set new_arc [list $newid $node_id($i) $node_id($j)]
		set arc_list [lappend arc_list $new_arc]
	    }
	}
    }
    if $DEBUG {
	puts " ...Done"
    }
}

proc Make_Random { } {
    global size create_props

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of nodes:" -width 20 -anchor w
    entry .params.size.field -textvar size -width 4 -relief sunken
    frame .params.proba
    label .params.proba.label -text "Edge probability:" -width 20 -anchor w
    entry .params.proba.field -textvar create_props(proba) \
	-width 12 -relief sunken

    pack .params.size.label .params.size.field -side left
    pack .params.proba.label .params.proba.field -side left
    pack .params.size .params.proba -side top -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n\create_random \$size"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

}

######################################################################
# Num2Col -
# Function converting an angle (measured in radians) into an integer
# in the hexadecimal format, between 00 and ff.
#
# Parameters:
# - Alpha	angle
#
proc Num2Col { Alpha } {
    global PI
    global DEBUG 


    set res [format "%02x" [expr int(floor(256*3*$Alpha/$PI))]]
    if { [string compare $res "100" ] == 0 } {
	set res "ff"
    }

    if $DEBUG { puts "Col: $Alpha -> $res " }

    return $res
}

######################################################################
# Create_Colors -
# Procedure to create a number of different colors using the "color
# circle". The colors are stored in the global array "color".
#
# Parameters:
# - number	the number of different colors to generate.
#
proc Create_Colors { number } {
    global PI color
    global DEBUG

    for { set i 0 } { $i < $number } { incr i 1 } {
	set Alpha [expr 2.0*$PI*$i/double($number)]

	if [expr $Alpha < $PI/3.0] {
	    set red "ff"
	    set green "00"
	    set blue [Num2Col $Alpha]
	} elseif [expr $Alpha < 2.0*$PI/3.0] {
	    set Alpha [expr $Alpha-$PI/3.0]
	    set red "ff"
	    set green [Num2Col $Alpha]
	    set blue "00"
	} elseif [expr $Alpha < $PI] {
	    set Alpha [expr $Alpha-2.0*$PI/3.0]
	    set red [Num2Col $Alpha]
	    set green "ff"
	    set blue "00"
	} elseif [expr $Alpha < 4.0*$PI/3.0] {
	    set Alpha [expr $Alpha-3.0*$PI/3.0]
	    set red "00"
	    set green "ff"
	    set blue [Num2Col $Alpha]
	} elseif [expr $Alpha < 5.0*$PI/3.0] {
	    set Alpha [expr $Alpha-4.0*$PI/3.0]
	    set red "00"
	    set green [Num2Col $Alpha]
	    set blue "ff"
	} else {
	    set Alpha [expr $Alpha-5.0*$PI/3.0]
	    set red [Num2Col $Alpha]
	    set green "00"
	    set blue "ff"
	}

	set Col "\#"
	append Col $red $green $blue

	if $DEBUG { puts $Col }

	set color([expr $i+1]) $Col
    }
}

######################################################################
# Browse_Colors -
# User interface for browsing the list of colors known to "showrgb".
#
# Parameters: none
#
proc Browse_Colors { } {
    global red green blue

    set input [open "| showrgb" r]
    set colors [split [read $input] \n]

    foreach descr $colors {
	for { set i 3 } { $i < [llength $descr] } { incr i } {
	    set name [lindex $descr $i]
	    set red($name)   [lindex $descr 0]
	    set green($name) [lindex $descr 1]
	    set blue($name)  [lindex $descr 2]
	}
    }

    set names [lsort [array names red]]

    if [catch {toplevel .color_browser}] { return }

    set b .color_browser

    frame $b.actions
    button $b.actions.ok -text "Select" \
	-command "if \[catch {.piece_edit.att.color.sample configure -background \[$b.list get \[$b.list curselection\]\]\n set cur_piece_props(color) \[$b.list get \[$b.list curselection\]\] \n Recolor_Current_Piece}\] { set cur_piece_props(color) \$default_piece_props(color)}\n destroy $b"
    button $b.actions.cancel -text "Cancel" \
	-command "destroy $b"
    pack $b.actions.ok -side left 
    pack $b.actions.cancel -side right
    pack $b.actions -side bottom

    wm title $b "Color Browser"

    listbox $b.list -height 30 -width 20 \
	-yscrollcommand [list $b.sy set]
    scrollbar $b.sy -orient vertical -command [list $b.list yview]
    pack $b.sy -side right -fill y
    pack $b.list -side left -fill both -expand true

   foreach nom $names {
	$b.list insert end $nom
    }

    bind $b.list <ButtonRelease-1> "catch {$b.list configure -selectbackground \[$b.list get \[$b.list curselection\]\]}"

}

######################################################################
# Choose_Colors -
# Function returning a color obtained from the user using the 
# tk_chooseColor widget.
#
# Parameters: none
#
proc Choose_Colors { } {
    global cur_piece_props

    if { [info tclversion] <= 7.5 } {
	ReportError "Color Choice" "Not available: you should upgrade your tcl
version!"
	return
    }

    set res [tk_chooseColor -initialcolor $cur_piece_props(color)]

    if { "" != $res } {
	set cur_piece_props(color) $res
	.piece_edit.att.color.sample configure \
	    -background $cur_piece_props(color)
	Recolor_Current_Piece
    }

}

######################################################################
# Get_Random_Color -
# Function returning a random color.
#
# Parameters: none
#
proc Get_Random_Color { } {
    global rand cur_piece_props

    if ![info exists rand] {
	randomInit [pid]
    }

    set R [expr int(256*[random])]
    set G [expr int(256*[random])]
    set B [expr int(256*[random])]

    set res [format "\#%02x%02x%02x" $R $G $B]

    set cur_piece_props(color) $res
    .piece_edit.att.color.sample configure -background $cur_piece_props(color)
    Recolor_Current_Piece

}
