#
# Module grouping the procedures for "quantitative" analysis of 
# heaps of pieces, excluding simulation to be found in "drop.tcl"
#
######################################################################
# Bound -
# Interface to the "tet_bound" application. 
# Note the call of write_tetris with the "hexa" flag to circumvent a 
# temporary inability of tet_bound to understand symbolic color names.
#
# Parameters: none
#
proc Bound { } {
    global KEEP

    set TMP_TET_FILE "/tmp/teted[pid].tet"
    set TMP_RES_FILE "/tmp/teted[pid].res"

    write_tetris $TMP_TET_FILE hexa

    exec tet_bound -f $TMP_TET_FILE -nmax 500 > $TMP_RES_FILE

    set resfile [open $TMP_RES_FILE r]
    set result [read $resfile]
    close $resfile

    LogResult "Bounds on the growth rate" "$result"

    if !$KEEP {
	RemoveFile $TMP_TET_FILE
	RemoveFile $TMP_RES_FILE
    }
}

######################################################################
# Bound_Graph -
# Interface to the "gr_bound" application. 
# The result of write_interaction_graph is checked because in the 
# case of "full menus", we may arrive here with a "non flat" heap of pieces.
#
# Parameters: none
#
proc Bound_Graph { } {
    global KEEP

    set TMP_GRAPH_FILE "/tmp/teted[pid].gr"
    set TMP_RES_FILE "/tmp/teted[pid].res"

    set code [write_interaction_graph $TMP_GRAPH_FILE]

    if { 0 == [string compare $code BAD_HEIGHT] } { return }

    exec gr_bound -f $TMP_GRAPH_FILE > $TMP_RES_FILE

    set resfile [open $TMP_RES_FILE r]
    set result [read $resfile]
    close $resfile

    LogResult "Bounds on the growth rate" "$result"

    if !$KEEP {
	RemoveFile $TMP_GRAPH_FILE
	RemoveFile $TMP_RES_FILE
    }
}

######################################################################
# Simulate -
# Dialog interface to let the user specify parameters for the 
# simulation of the Tetris game.
#
# Parameters: none
#
proc Simulate { } {

    if [catch {toplevel .params}] { return }

    wm title .params "Simulation Parameters"

    frame .params.simlength
    label .params.simlength.label -text "Simulation Length" \
	-width 15 -anchor w
    entry .params.simlength.field -textvar sim_length -width 8 \
	-relief sunken
    
    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n Run_Simulation \$sim_length"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"
    
    pack .params.simlength.label .params.simlength.field -side left \
	-fill x
    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.simlength -side top -fill x
    pack .params.actions -side top
    
    bind .params <Return> {.params.actions.ok invoke}
    bind .params <Escape> {.params.actions.cancel invoke}

    grab .params
}

######################################################################
# Run_Simulation -
# Interface to the call of "mat_sim" which simulates the Tetris game.
# The application is called with its option -compile in order to
# speed up **long** simulations.
#
# Parameters: 
# - sim_length	the number of pieces to drop
#
proc Run_Simulation { sim_length } {
    global KEEP

    set TMP_MAT_FILE "/tmp/teted[pid].mat"
    set TMP_RES_FILE "/tmp/teted[pid].res"

    Write_MaxPlus_Matrices $TMP_MAT_FILE bruno "A"
    if $KEEP {
	set opts "-nbevt $sim_length -compile -keep"
    } else {
	set opts "-nbevt $sim_length -compile"
    }
    exec mat_sim -f $TMP_MAT_FILE $opts \
	| grep gamma > $TMP_RES_FILE 2> /dev/null

    set resfile [open $TMP_RES_FILE r]
    set result [read $resfile]
    close $resfile

    LogResult "Simulation Result" "$result"

    if !$KEEP {
	RemoveFile $TMP_MAT_FILE
    }
}
