#
# Procedures concerning sequences of pieces and their execution.
#

proc Make_Sequence_Editor { } {
    global state file_name

    if [catch {toplevel .seq_edit}] { return }

    wm title .seq_edit "Sequence Editor"

    # create the top menu
    set tm [frame .seq_edit.topmenu -relief raised \
		-class MenFrame -borderwidth 2]
    pack $tm -side top -fill x

    # menu for creation and manipulation
    menubutton $tm.file -text "Sequence" \
	-menu $tm.file.menu \
	-underline 0
    set m [menu $tm.file.menu -tearoff 1]
    $m add command -label "New" -command "New_Sequence" \
	-underline 0
    $m add separator
    $m add command -label "Save" -command "Save_Sequence .seq_edit.list 0" \
	-underline 0
    $m add command -label "Save Expanded" \
	-command "Save_Sequence .seq_edit.list 1" \
	-underline 5
    $m add command -label "Load" -command "Load_Sequence .seq_edit.list" \
	-underline 0
    $m add separator
    $m add command -label "Dismiss" -command "Quit_Sequence_Editor" \
	-underline 0
    pack $tm.file -side left

    # menu for editin functions
    menubutton $tm.edit -text "Edit" \
	-menu $tm.edit.menu \
	-underline 0
    set m [menu $tm.edit.menu -tearoff 1]
    $m add command -label "New" -command "New_Sequence" \
	-underline 0
    $m add command -label "Clean" -command "Clean_Sequence .seq_edit.list" \
	-underline 0
    $m add command -label "Expand" -command "Expand_Sequence .seq_edit.list" \
	-underline 0
    pack $tm.edit -side left

    # menu for dropping utilities
    menubutton $tm.drop -text "Drop" -menu $tm.drop.menu \
	-underline 0
    set m [menu $tm.drop.menu -tearoff 1]
    $m add command -label "Drop Once" -command "Drop_Sequence" \
	-underline 0
    $m add command -label "Tetris Game" \
	-command "Tetris_Game_Interface sequence" \
	-underline 0
    pack $tm.drop -side left

    # menu for solution methods
    menubutton $tm.quanti -text "Solve" -menu $tm.quanti.menu \
	-underline 0
    set m [menu $tm.quanti.menu -tearoff 1]
    $m add command -label "Cycle Time" -command "Sequence_Cycle_Time" \
	-underline 0
    $m add command -label "Tetris Game" \
	-command "Tetris_Game_Interface sequence" \
	-underline 0
    pack $tm.quanti -side left

    #menu for translations in various formats
    menubutton $tm.export -text "Export" -menu $tm.export.menu \
	-underline 1
    set m [menu $tm.export.menu -tearoff 0 -disabledforeground skyblue]
    $m add command -label "(max,+) matrix" \
	-command "Gen_MaxPlus_Matrices [Make_File_Name $file_name mat] sequence" \
	-underline 1
    pack $tm.export -side left

    # field for the sequence
    entry .seq_edit.list -width 50 -xscrollcommand [list .seq_edit.sx set] 
    scrollbar .seq_edit.sx -orient horizontal \
	-command [list .seq_edit.list xview]
    pack .seq_edit.list .seq_edit.sx -side top -fill x -expand true

    bind .seq_edit.list <KeyPress> "set state(.seq) modified"

    if [winfo exists .tg_params] {
	.tg_params.mode.sequence configure -state normal
    }

    set state(.seq) saved
}

######################################################################
# Quit_Sequence_Editor - 
# The procedure to check that it is ok to leave, and destroy the 
# editor of dropping sequences.
#
# Parameters: none
#
proc Quit_Sequence_Editor { } {
    global sim_mode state

    if { "$state(.seq)" != "saved" } {
	switch [tk_dialog .quit "Modified" "The sequence has been modified. Do you wish to save it?" {warning} 0 {Save & Quit} {Quit} {Cancel}] {
	    0 { 
		if ![Save_Sequence .seq_edit.list 0] { return }
	    }
	    1 { }
	    2 { return }
	}
    }

    if [winfo exists .tg_params] {
	.tg_params.mode.sequence configure -state disabled
	set sim_mode random
    }

    destroy .seq_edit
}

proc New_Sequence { } {
    global state

    if { "$state(.seq)" != "saved" } {
	switch [tk_dialog .error "Modified" "The sequence has been modified. Do you wish to save it?" {warning} 0 {Save & New} {New} {Cancel}] {
	    0 { 
		if ![Save_Sequence .seq_edit.list 0] { return }
	    }
	    1 { }
	    2 { return }
	}
    }

    .seq_edit.list delete 0 end
    set state(.seq) saved
}

proc Get_And_Display_Piece_Number { ypos } {
    global DEBUG

    if $DEBUG {
	puts "Find and Display Number of Piece at $ypos $copy"
    }

    if ![winfo exists .seq_edit] { return }

    set number [Find_Piece $ypos]

    if { 0 < $number } {
	.seq_edit.list insert end " $number"
	if $DEBUG {
	    puts "Adding piece \# $number"
	}
    } else {
	if $DEBUG {
	    puts "none found"
	}
    }
}

# a procedure to clean a string and put it in "normal" form
proc Clean_String { seq } {
    
    set res $seq
    regsub -all ">" $res " > " res
    regsub -all "<" $res " < " res
    regsub -all "n" $res " n " res
    regsub -all {\(} $res " \( " res
    regsub -all {\)[ ]*\^} $res " \)\^ " res
    regsub -all {[  ]+} $res " " res
    set res [string trim $res]
    return $res
}

proc Expand_String { sequence } {
    global cur_piece_nb
    global DEBUG

    set seq [Clean_String $sequence]
    set state 0
    set curr nil
    set level 0
    set subseq(0) {}

    foreach token [split $seq] {

	if $DEBUG {
	    puts "Found Token $token (current=$curr, state=$state, level=$level)"
	}
	if { $token == "n" } {
	    set token $cur_piece_nb
	}
	switch -regexp $token {
	    ">" {
		if { $curr != "nil" } {
		    set state decr
		}
	    }
	    "<" {
		if { $curr != "nil" } {
		    set state incr
		}
	    }
	    {\(} {
		incr level 1
		set subseq($level) {}
		set state 0
	    }
	    {\)\^} {
		set state mult
	    }
	    {[0-9]+} {	
		switch $state {
		    incr {
			if { $token >= $curr } {
			    for { set i [expr $curr+1] } { $i < $token } { incr i 1 } {
				lappend subseq($level) $i
			    }
			} else {
			    for { set i [expr $curr+1] } { $i <= $cur_piece_nb } { incr i 1 } {
				lappend subseq($level) $i
			    }
			    for { set i 1 } { $i < $token } { incr i 1 } {
				lappend subseq($level) $i
			    }
			}
		    }
		    decr {
			if { $token <= $curr } {
			    for { set i [expr $curr-1] } { $i > $token } { incr i -1 } {
				lappend subseq($level) $i
			    }
			} else {
			    for { set i [expr $curr-1] } { $i > 0 } { incr i -11 } {
				lappend subseq($level) $i
			    }
			    for { set i $cur_piece_nb } { $i > $token } { incr i -1 } {
				lappend subseq($level) $i
			    }
			}
		    }
		    mult {
			for { set i 0 } { $i < $token } { incr i 1 } {
			    lappend subseq([expr $level-1]) \
				[join $subseq($level)]
			}
		    }
		    0 {
			# nothing particular
		    }
		}

		if { $state != "mult" } {
		    lappend subseq($level) $token
		    set curr $token
		} else {
		    # current token should be the last regular number read
		    incr level -1
		}
		set state 0
	    }
	    .* {
		puts "Illegal token $token"
	    }
	}
    }

    set res [join $subseq(0)]
    if $DEBUG {
	puts "Expand = $res"
    }
    return $res
}

proc Clean_Sequence { win } {

    set propre [Clean_String [$win get]]
    $win delete 0 end
    $win insert end $propre
}

proc Expand_Sequence { win } {

    set full [Expand_String [$win get]]
    $win delete 0 end
    $win insert end $full

}

proc Drop_Sequence { } {
    global cur_piece_nb
    global DEBUG

    foreach num [split [Expand_String [.seq_edit.list get]]] {
	if { $num <= $cur_piece_nb } {
	    if $DEBUG {
		puts "Dropping piece $num"
	    }
	    Drop_Piece $num
	}
    }
}

proc Save_Sequence { win expand } {
    global state seq_file_name

    set types {
	{{Sequence File}       {.seq}  }
	{{All Files}        *             }
    }

#     set seq_file_name [tk_getSaveFile -defaultextension seq \
# 		       -initialfile [file tail $seq_file_name] \
# 		       -filetypes $types -parent .]
    set options "-defaultextension seq -initialfile {[file tail $seq_file_name]} -filetypes {$types} -parent ."

    set seq_file_name [GetSaveFile $options]

    if { $seq_file_name != "" } {
	Write_Sequence $win $seq_file_name $expand
	set state(.seq) saved
	return 1
    } else {
	return 0
    }
}

proc Browse_Sequence_File { } {
    global seq_file_name

    set types {
	{{Sequence File}       {.seq}  }
	{{All Files}        *             }
    }

#     set new_file_name [tk_getOpenFile -defaultextension teted \
# 			   -initialfile [file tail $file_name] \
# 			   -filetypes $types -parent .]
    set options "-defaultextension seq -initialfile {[file tail $seq_file_name]} -filetypes {$types} -parent ."

    set new_file_name [GetOpenFile $options]
    
    if { $new_file_name != "" } {
	set seq_file_name $new_file_name
    }
}

######################################################################
# Load_Sequence -
# A user dialog interface for specifying a sequence file to load into
# the editor. 
# 
# Parameters:
# - win		the entry widget into which to load
#
proc Load_Sequence { win } {
    global seq_file_name

    if [catch {toplevel .params}] { return }

    wm title .params "Load Sequence"

    frame .params.filename
    label .params.filename.label -text "File Name" -width 15 -anchor w
    entry .params.filename.field -textvar seq_file_name -width 40 \
	-relief sunken
    pack .params.filename.label -side left
    pack .params.filename.field -side left -expand true -fill x

    # frame with buttons. 
    frame .params.actions
    button .params.actions.new -text "Replace" \
	-underline 0 \
	-command "destroy .params\n Read_Sequence $win \$seq_file_name new"
    button .params.actions.at_start -text "At Start" \
	-underline 3 \
	-command "destroy .params\n Read_Sequence $win \$seq_file_name start"
    button .params.actions.at_end -text "At End" \
	-underline 3 \
	-command "destroy .params\n Read_Sequence $win \$seq_file_name end"
    button .params.actions.at_cursor -text "At Cursor" \
	-underline 3 \
	-command "destroy .params\n Read_Sequence $win \$seq_file_name cursor"
    button .params.actions.browse -text "Browse" \
	-underline 0 \
	-command "Browse_Sequence_File"
    button .params.actions.cancel -text "Cancel" \
	-underline 0 \
	-command "destroy .params"
    pack .params.actions.new .params.actions.at_start \
	.params.actions.at_end .params.actions.at_cursor \
	.params.actions.browse .params.actions.cancel -side left 

    bind .params <Return> {.params.actions.new invoke}
    bind .params <Escape> {.params.actions.cancel invoke}

    pack .params.filename -side top -expand true -fill x
    pack .params.actions -side top

    grab .params

}

proc Write_Sequence { win file_name expand } {

    set outfile [open $file_name w]

    # writing out expanded sequence because the analysis programs 
    # may not know to do it.
    if $expand {
	puts $outfile [Expand_String [$win get]]
    } else {
	puts $outfile [Clean_String [$win get]]
    }

    close $outfile
}

proc Read_Sequence { win file_name merge_mode } {
    global DEBUG

    if [catch {open $file_name r} infile] {
	ReportError "Load Error" \
	    "Could not open file $file_name for reading"
	return 
    }

    if ![winfo exists .seq_edit] { return }

    gets $infile line

    # put what is read directly without checking...
    switch $merge_mode {
	new {
	    $win delete 0 end
	    $win insert 0 $line
	}
	start {
	    $win insert 0 "$line "
	}
	end {
	    $win insert end " $line"
	}
	cursor {
	    $win insert insert " $line "
	}
    }

    close $infile
}

######################################################################
# Compute_And_Write_Matrix -
# Interface procedure to "mat_sim" for computing and saving to a file
# the (max,+) matrix that corresponds to a sequence of pieces.
#
# Parameters:
# - file_name	the file into which to save
# - format	the matrix format (scilab|standard|sparse)
# - prefix	the name of the matrix (for the Scilab format)
#
proc Compute_And_Write_Matrix { file_name format prefix } {
    global KEEP

    set TMP_MAT_FILE "/tmp/teted[pid].mat"
    set TMP_SEQ_FILE "/tmp/teted[pid].seq"

    puts "format = $format"

    Write_MaxPlus_Matrices $TMP_MAT_FILE standard 0
    Write_Sequence .seq_edit.list $TMP_SEQ_FILE 1
    
    switch $format {
	scilab {
	    exec mat_sim -f $TMP_MAT_FILE -seq $TMP_SEQ_FILE -o $file_name \
		-scilab -prefix $prefix
	}
	sparse {
	    exec mat_sim -f $TMP_MAT_FILE -seq $TMP_SEQ_FILE -o $file_name \
		-sparse
	}
	normal {
	    exec mat_sim -f $TMP_MAT_FILE -seq $TMP_SEQ_FILE -o $file_name
	}
    }	    

    if !$KEEP {
	RemoveFile $TMP_MAT_FILE
	RemoveFile $TMP_SEQ_FILE
    }
}

proc Sequence_Cycle_Time { } {
    global KEEP DEBUG

    set TMP_MAT_FILE "/tmp/teted[pid]-seq.mat"
    set TMP_RES_FILE "/tmp/teted[pid]-seq.res"

    Compute_And_Write_Matrix $TMP_MAT_FILE normal "A"
    exec scscyc -f $TMP_MAT_FILE -o $TMP_RES_FILE

    set resfile [open $TMP_RES_FILE r]
    set result [read $resfile]
    close $resfile 

    set seq_length [llength [Expand_String [.seq_edit.list get]]]

    if [regexp {.*cycle mean: ([0-9]*)[ ]*/[ ]*([0-9]*).*} \
	    $result match numer denom] {
	if $DEBUG {
	    puts "MA=$match NU=$numer DE=$denom"
	}
	set result "Cycle time = $numer / [expr $denom*$seq_length]\n"
    } else {
	set result "Unable to compute"
    }

    LogResult "Analysis Results" "$result"

    if !$KEEP {
	RemoveFile $TMP_MAT_FILE
	RemoveFile $TMP_RES_FILE
    }
}
